SPACES_GAP_FOR_FREQUENCY = "          "
SPACES_GAP_BETWEEN_TOKENS = "  "
TARGET_LINE_LENGHT = 60
SPACES_LEVEL_INDENTATION = "   "
COMMENT_INI = "# "
POSITIVE_CLOSURE = "+"
KLEENE_CLOSURE = "*"
OPT_CARDINALITY = "?"
SHAPE_LINK_CHAR = "@"
ANNOTATION_BEGIN = "//"